const db = require("../services/dbAccess.services");
const os = require("os");
const vers = require("../configs/version.configs");

import { logger } from "../utils/logger.utils";

async function buildJson() {
  let json = {};
  json.count = {};
  // Async call all db queries
  const userCount = db.colCount("users");
  const regCount = db.colCount("registrations");
  const pisCount = db.colCount("pis");
  const corpusCount = db.colCount("corpus");
  const stationsCount = db.colCount("stations");
  const timetableCount = db.colCount("timetable");

  // Insert data
  json.mode = process.env.NODE_ENV;
  json.host = os.hostname();
  // Await and insert async calls
  json.count.users = await userCount;
  json.count.reg = await regCount;
  json.count.pis = await pisCount;
  json.count.corpus = await corpusCount;
  json.count.stations = await stationsCount;
  json.count.timetable = await timetableCount;
  return json;
}

async function hits() {
  logger.debug("statsServices.hits: Statistics Requested");
  const out = await buildJson();
  return out;
}

async function getVersions() {
  logger.debug("statsServices.getVersions: Fetching versions");
  const mqClt = await db.query("versions", { target: "timetable-mgr" });
  const data = {
    backend: vers.app,
    "mq-client": mqClt[0]?.["version"] || "",
  };
  return data;
}

async function statistics() {
  logger.debug("statsServices.statistics: Fetching statistics");

  const timetablePromise = db.query("meta", { type: "CifMetadata" });
  const pisPromise = db.query("meta", { type: "PisMetadata" });
  const corpusPromise = db.query("meta", { target: "corpus" });
  const stationsPromise = db.query("meta", {type: "StationsMetadata"});

  const lengthUsersPromise = db.colCount("users");
  const lengthRegistrationsPromise = db.colCount("registrations");
  const lengthCorpusPromise = db.colCount("corpus");
  const lengthStationsPromise = db.colCount("stations");
  const lengthPisPromise = db.colCount("pis");
  const lengthTimetablePromise = db.colCount("timetable");
  const lengthReasonCodesPromise = db.colCount("reasonCodes");

  const [
    timetable,
    pis,
    corpus,
    reasonCodes,
    lengthUsers,
    lengthRegistrations,
    lengthCorpus,
    lengthStations,
    lengthPis,
    lengthTimetable,
    stations,
  ] = await Promise.all([
    timetablePromise,
    pisPromise,
    corpusPromise,
    lengthUsersPromise,
    lengthRegistrationsPromise,
    lengthCorpusPromise,
    lengthStationsPromise,
    lengthPisPromise,
    lengthTimetablePromise,
    lengthReasonCodesPromise,
    stationsPromise,
  ]);

  return {
    hostname: os.hostname() || "Unknown",
    runtimeMode: process.env.NODE_ENV || "Unknown",
    updateTimes: {
      timetable: (timetable[0]["lastUpdate"]),
      pis: pis[0]["lastUpdate"],
      corpus: corpus[0]["updated_time"],
      stations: stations[0]["lastUpdate"],
    },
    dbLengths: {
      users: lengthUsers,
      registrations: lengthRegistrations,
      corpus: lengthCorpus,
      stations: lengthStations,
      pis: lengthPis,
      timetable: lengthTimetable,
    },
  };
}

module.exports = {
  hits,
  statistics,
  getVersions,
};
